// Char8ArrayMessage.cc
/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. MPF is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>

#include "DLList.h"
#include "DataFreeList.h"
#include "Char8ArrayMessage.h"

DLList* Char8ArrayMessage::pFreeList = 0;

int32 Char8ArrayMessage:: toBuffer(char **buffer) const
{
    int32 nbytes;

    nbytes = StandardFieldsMessage::toBuffer(buffer);
    nbytes += MPF::toBuffer(buffer,size);
    nbytes += MPF::toBuffer(buffer,(const void *)value,size);
    nbytes += MPF::toBuffer(buffer,numberRetrys);
    nbytes += MPF::toBuffer(buffer,eomLen);
    nbytes += MPF::toBuffer(buffer,eomString[0]);
    nbytes += MPF::toBuffer(buffer,eomString[1]);
    return(nbytes);
}

int32 Char8ArrayMessage:: fromBuffer(const char **buffer)
{
    int32 nbytes=0;

    nbytes = StandardFieldsMessage::fromBuffer(buffer);
    nbytes += MPF::fromBuffer(buffer,&size);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Char8ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    nbytes += MPF::fromBuffer(buffer,(void *)value,size);
    nbytes += MPF::fromBuffer(buffer,&numberRetrys);
    nbytes += MPF::fromBuffer(buffer,&eomLen);
    nbytes += MPF::fromBuffer(buffer,&eomString[0]);
    nbytes += MPF::fromBuffer(buffer,&eomString[1]);
    return(nbytes);
}

int32 Char8ArrayMessage:: fromBufferSwitch(const char **buffer)
{
    int32 nbytes=0;

    nbytes = StandardFieldsMessage::fromBufferSwitch(buffer);
    nbytes += MPF::fromBuffer(buffer,&size);
    if(maxSize==0) allocValue(size);
    if(maxSize<size) {
	printf("Char8ArrayMessage:: fromBuffer maxSize<size VERY bad\n");
	exit(-1);
    }
    nbytes += MPF::fromBuffer(buffer,(void *)value,size);
    nbytes += MPF::fromBufferSwitch(buffer,&numberRetrys);
    nbytes += MPF::fromBufferSwitch(buffer,&eomLen);
    nbytes += MPF::fromBufferSwitch(buffer,&eomString[0]);
    nbytes += MPF::fromBufferSwitch(buffer,&eomString[1]);
    return(nbytes);
}

void Char8ArrayMessage:: print() const
{
    StandardFieldsMessage::print();
    printf("numberRetrys:%d ",(int)numberRetrys);
    printf("eomLen:%d ",(int)eomLen);
    printf("eomString:%2.2x %2.2x ",eomString[0],eomString[1]);
    printf("maxSize:%d ",maxSize);
    printf("size:%d\n",size);
    printf("pvalue:%p value ",value);
    for(int i=0; i<size; i++) printf(" %2.2x",((int)value[i])&0xff);
    printf("| %s|\n",value);
    printf("\n");
}

void *Char8ArrayMessage:: operator new(size_t size)
{
    if(size==sizeof(Char8ArrayMessage)) return MPF:: allocate(&pFreeList,size);
    printf("Char8ArrayMessage new size !=sizeof(Char8ArrayMessage)\n");
    return(0);
}
 
void Char8ArrayMessage:: operator delete(void *p, size_t size)
{
    if(size==sizeof(Char8ArrayMessage)) {
	MPF::free(pFreeList,p,size);
	return;
    }
    printf("Char8ArrayMessage delete size !=sizeof(Char8ArrayMessage)\n");
}

void Char8ArrayMessage::setSize(int32 size)
{
    if(size>maxSize) {
	printf("Char8ArrayMessage::setSize size %d > maxSize %d\n",
	    size,maxSize);
	this->size = maxSize;
    } else {
	this->size = size;
    }
}

void Char8ArrayMessage::allocValue(int32 maxSize)
{
    DataFreeList::alloc(maxSize,(void **)&value,&this->maxSize);
}

void Char8ArrayMessage::freeValue()
{
    if(maxSize==0) return;
    DataFreeList::free((void *)value,maxSize);
    maxSize=0; value=0;
}
